package analyzer;

/**
 * Class that describes a Lexical component  
 * @author Sergio Alcocer
 *
 */
public class Token implements TokenConstants{
	
	public static final int EOF = 0;
	
	private int kind;
	
	private String lexeme;
	
	private int row;
	
	private int column;
	

	/**
	 * Constructor
	 * @param kind Lexical category identifier
	 * @param lexeme lexeme that generates the component
	 * @param row Row in which this component starts
	 * @param column Column in which this component starts
	 */
	public Token(int kind, String lexeme, int row, int column) {
		this.kind = kind;
		this.lexeme = lexeme;
		this.row = row + 1;
		this.column = column + 1;
	}
	
	/**
	 * Gets the category identifier
	 * @return component type
	 */
	public int getKind() {
		return this.kind;
	}
	
	/**
	 * Gets the lexeme
	 * @return component's lexeme
	 */
	public String getLexeme() {
		return this.lexeme;
	}
	
	/**
	 * Gets starting row
	 * @return starting row of the component
	 */
	public int getRow() {
		return this.row;
	}
	
	/**
	 * Gets starting column
	 * @return starting column of the component
	 */
	public int getColumn() {
		return this.column;
	}
	
	/**
	 * Gets Token description
	 * @return token description
	 */
	public String toString() {
		return "[Row: "+row+"][Column: "+column+"][Kind: "+kind+"] "+lexeme;
	}
}
