package ast;

import java.util.ArrayList;

/**
 * Class that stores class information, such as attributes, methods and constructors
 * @author Sergio Alcocer
 *
 */
public class Class extends Elem {
	
	private ArrayList<Attribute> attributes;
	private ArrayList<Method> methods;
	private ArrayList<Method> constructors;
	
	/**
	 * Constructor of the class
	 */
	public Class(){
		super(T_CLASS);
		attributes = new ArrayList<Attribute>();
		methods = new ArrayList<Method>();
		constructors = new ArrayList<Method>();
	}
	
	/**
	 * Adds an attribute to the list
	 * @param _att Attribute to be added
	 */
	public void addAttribute(Attribute _att){
		attributes.add(_att);
	}
	
	/** Adds a method to the list
	 * @param _method Method to be added (from ast.Method)
	 */
	public void addMethod(Method _method){
		if (!_method.isConstructor())methods.add(_method);
		else constructors.add(_method);
	}
	
	/**
	 * Gets the constructors of the class
	 * @return ArrayList&lt;Method&gt; with all the constructos
	 */
	public ArrayList<Method> getConstructors(){
		return constructors;
	}
	
	/**
	 * Gets the mehtods of the class 
	 * @return ArrayList&lt;Method&gt; with all the methods
	 */
	public ArrayList<Method> getMethods(){
		return methods;
	}
}
