package generator;

import java.util.ArrayList;

/**
 * Class that stores the information of an object to be created
 * @author Sergio Alcocer
 */
public class ObjectCreated {
	private String objectName; 
	private ArrayList<String> constructorParameters; 
	
	/**
	 * Constructor of the class
	 * @param objectName name of the object that is being created
	 * @param constructorParameters ArrayList&lt;String&gt; with the parameters code
	 */
	public ObjectCreated(String objectName, ArrayList<String>constructorParameters){
		this.objectName = objectName;
		this.constructorParameters = constructorParameters;
	}

	/**
	 * Gets the object's name
	 * @return object's name
	 */
	public String getObjectName(){
		return objectName;
	}
	
	/**
	 * Returns a representation of how the object is created to be shown in the list
	 * @return String with the representation of the object
	 */
	public String toString(){
		String returned = new String("( ");
		for (int i = 0; i < constructorParameters.size(); i++){
			returned += constructorParameters.get(i) + ", ";
		}
		
		if (returned.charAt(returned.length()-2) == ','){
			returned = returned.substring(0,returned.length()-2);
		}
		
		returned += " )";
		return returned;
	}
	
	/**
	 * Gets an array with the parameters' code
	 * @return ArrayList&lt;String&gt; of parameters' code
	 */
	public ArrayList<String> getParams(){
		return constructorParameters;
	}
}
